/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <stdio.h>
#include <fcntl.h>
#include <tiuser.h>
#include <string.h>
#include <unistd.h>

extern int t_errno;
extern char *t_errlist[];

extern void log(const char *, ...);
extern int initserver(char *, char *, uint_t, uint_t);

#define tlog(STR) log("%s: %s", (STR), t_errlist[t_errno])

void
rwhod(char *transport, char *addr, uint_t alen)
{
	int fd;
	struct t_unitdata *udp;
	FILE *fp;
	int flags;

	/*
	 * Create a server-style transport endpoint.
	 */
	fd = initserver(transport, addr, alen, 0);

	/*
	 * Allocate the t_unitdata structure to be used to
	 * send and receive data.
	 */
	udp = (struct t_unitdata *)t_alloc(fd, T_UNITDATA,
	  T_ALL);
	if (udp == NULL) {
		tlog("Cannot allocate unitdata");
		return;
	}

	/*
	 * Receive request messages, execute the command, and
	 * send the output back to the client.
	 */
	for ( ; ; ) {
		if (t_rcvudata(fd, udp, &flags) < 0) {
			if (t_errno == TLOOK) {
				/*
				 * Try to retrieve the error
				 * indication.
				 */
				if (t_rcvuderr(fd, NULL) < 0) {
					tlog("Cannot clear TLOOK");
					break;
				}
			} else {
				tlog("Cannot receive data");
			}
			break;
		} else {
			/*
			 * Execute the command.  fp is open to the
			 * command's standard output.
			 */
			fp = popen(udp->udata.buf, "r");
			if (fp) {
				/*
				 * Send the output of the command back to
				 * the client.
				 */
				while (fgets(udp->udata.buf,
				  udp->udata.maxlen, fp)) {
					udp->udata.len =
					  strlen(udp->udata.buf);
					if (t_sndudata(fd, udp) < 0) {
						tlog("Cannot send data");
						return;
					}
				}
				pclose(fp);

				/*
				 * Send final message.  Format is one
				 * byte containing 0.
				 */
				udp->udata.len = 1;
				*udp->udata.buf = '\0';
				if (t_sndudata(fd, udp) < 0) {
					tlog("Cannot send data");
					break;
				}
			} else {
				log("Cannot exec command");
			}
		}
	}
}
